const db_con = require("../../../config/database-connection");

// ! Insert branch data query
const createBranch = (req, res) => {
  const {
    brach_name,
    brach_contact,
    brach_header,
    brach_footer,
    brach_location,
    brach_email,
    brach_fax,
    brach_web,
    brach_business_name,
  } = req.body;

  // ? Validate required fields
  if (
    !brach_name ||
    !brach_contact ||
    !brach_header ||
    !brach_footer ||
    !brach_location ||
    !brach_email ||
    !brach_fax ||
    !brach_web ||
    !brach_business_name
  ) {
    return res.status(400).json({ error: "All fields are required" });
  }

  // ? Get the max from existing branches
  const queryMaxId =
    "SELECT MAX(CAST(SUBSTRING(brach_id,2) AS UNSIGNED)) AS max_id FROM sys_branch_tbl";
  db_con.query(queryMaxId, (error, results) => {
    if (error) {
      return res.status(500).send(error);
    }

    const maxID = results[0].max_id || 0;
    const nextID = maxID + 1;

    // ? Format the next branch id
    const brach_id = "B" + `${String(nextID).padStart(2, "0")}`;

    // ? Check if the branch name already exists
    const queryCheckName =
      "SELECT COUNT(*) AS count FROM sys_branch_tbl WHERE brach_name = ?";
    db_con.query(queryCheckName, [brach_name], (checkError, checkResults) => {
      if (checkError) {
        return res.status(500).send(checkError);
      }

      const nameExists = checkResults[0].count > 0;

      if (nameExists) {
        return res.status(400).json({ error: "Branch name already exists" });
      }

      // ? Insert branch data
      const queryInsert =
        "INSERT INTO sys_branch_tbl (brach_id, brach_name, brach_contact, brach_header, brach_footer, brach_location, brach_email, brach_fax, brach_web, brach_business_name) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)";
      const values = [
        brach_id,
        brach_name,
        brach_contact,
        brach_header,
        brach_footer,
        brach_location,
        brach_email,
        brach_fax,
        brach_web,
        brach_business_name,
      ];

      db_con.query(queryInsert, values, (insertError, insertResults) => {
        if (insertError) {
          res.status(500).send(insertError);
        } else {
          res.status(201).json({
            message: "Branch added successfully",
            branchId: brach_id,
          });
        }
      });
    });
  });
};

// ! Update branch data query
const updateBranch = (req, res) => {
  const branchID = req.params.id;
  const {
    brach_name,
    brach_contact,
    brach_header,
    brach_footer,
    brach_location,
    brach_email,
    brach_fax,
    brach_web,
    brach_business_name,
  } = req.body;

  // ? Update branch details
  const queryUpdate =
    "UPDATE sys_branch_tbl SET brach_name = ?, brach_contact = ?, brach_header = ?, brach_footer = ?, brach_location = ?, brach_email = ?, brach_fax = ?, brach_web = ?, brach_business_name = ? WHERE brach_id = ?";
  const values = [
    brach_name,
    brach_contact,
    brach_header,
    brach_footer,
    brach_location,
    brach_email,
    brach_fax,
    brach_web,
    brach_business_name,
    branchID,
  ];

  db_con.query(queryUpdate, values, (error, results) => {
    if (error) {
      res.status(500).send(error);
    } else {
      if (results.affectedRows > 0) {
        res.status(200).json({ message: "Branch updated successfully" });
      } else {
        res.status(404).json({ message: "Branch not found" });
      }
    }
  });
};

// ! Get all branch data query (table view)
const getAllBranches = (req, res) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;

    const selectQuery = `
          SELECT brach_id, brach_name, brach_contact, brach_header, brach_footer, brach_location, brach_email, brach_fax, brach_web, brach_business_name
          FROM sys_branch_tbl
          WHERE brach_name LIKE ? OR brach_id LIKE ? OR brach_contact LIKE ?
          ORDER BY ${orderColumn} ${orderDirection}
          LIMIT ?, ?
        `;

    const countQuery = `
          SELECT COUNT(brach_id) AS total FROM sys_branch_tbl
          WHERE brach_name LIKE ? OR brach_id LIKE ? OR brach_contact LIKE ?
        `;

    db_con.query(
      selectQuery,
      [searchPattern, searchPattern, searchPattern, offset, limit],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        // ? Check if there's no data in the table
        if (results.length === 0) {
          return res.status(200).json({
            draw: +draw,
            recordsTotal: 0,
            recordsFiltered: 0,
            data: [],
            message: "No data found in the table.",
          });
        }

        db_con.query(
          countQuery,
          [
            searchPattern,
            searchPattern,
            searchPattern,
            searchPattern,
            searchPattern,
          ],
          (countError, countResults) => {
            if (countError) {
              return res.status(500).send(countError);
            }

            const totalRecords = countResults[0].total;
            const response = {
              draw: +draw,
              recordsTotal: totalRecords,
              recordsFiltered: totalRecords,
              data: results,
            };

            res.status(200).json(response);
          }
        );
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

// ! Get branch data query by selected branch id
const getBranchById = (req, res) => {
  const branchID = req.params.id;

  // ? Branch data get query
  db_con.query(
    "SELECT * FROM sys_branch_tbl WHERE brach_id = ?",
    [branchID],
    (error, results) => {
      if (error) {
        res.status(500).send(error);
      } else {
        if (results.length === 0) {
          res
            .status(404)
            .json({ message: "No data found for the specified branch ID" });
        } else {
          res.status(200).json(results[0]);
        }
      }
    }
  );
};

// ! Get all branch names query
const getAllBranchName = (req, res) => {
  // ? Branch data get query
  db_con.query(
    "SELECT brach_id,brach_name FROM sys_branch_tbl",
    (error, results) => {
      if (error) {
        res.status(500).send(error);
      } else {
        if (results.length === 0) {
          res
            .status(404)
            .json({ message: "No data found for the specified branch ID" });
        } else {
          res.status(200).json(results);
        }
      }
    }
  );
};

module.exports = {
  createBranch,
  updateBranch,
  getAllBranches,
  getBranchById,
  getAllBranchName,
};
