const db_con = require("../../../config/database-connection");
const insertPlAcc = require("../../../Function/pl-acc-function");

// ! insert doctor data query
const createDoctor = (req, res) => {
  const {
    doct_name,
    doct_location,
    doct_contact,
    doct_email,
    doct_commission_rate,
    doct_other_commission_rate,
  } = req.body;

  // ? Validate required fields
  if (!doct_name || !doct_location || !doct_contact) {
    return res.status(400).json({ error: "All fields are required" });
  }

  // ? Check if the doctor name already exists
  const queryCheckName =
    "SELECT COUNT(*) AS count FROM sys_doctor_tbl WHERE doct_name = ?";
  db_con.query(queryCheckName, [doct_name], (checkError, checkResults) => {
    if (checkError) {
      return res.status(500).send(checkError);
    }

    const nameExists = checkResults[0].count > 0;

    if (nameExists) {
      return res.status(400).json({ error: "Doctor name already exists" });
    }

    // ? Get the max from existing doctors
    const queryMaxId =
      "SELECT MAX(CAST(SUBSTRING(doct_id,2) AS UNSIGNED)) AS max_id FROM sys_doctor_tbl";
    db_con.query(queryMaxId, (error, results) => {
      if (error) {
        return res.status(500).send(error);
      }

      const maxID = results[0].max_id || 0;
      const nextID = maxID + 1;

      // ? Format the next doctor id
      const doct_id = "D" + `${String(nextID).padStart(4, "0")}`;

      // ? Insert doctor data
      const queryInsert =
        "INSERT INTO sys_doctor_tbl (doct_id, doct_name, doct_location, doct_contact, doct_email, doct_commission_rate, doct_other_commission_rate) VALUES (?, ?, ?, ?, ?, ?, ?)";
      const values = [
        doct_id,
        doct_name,
        doct_location,
        doct_contact,
        doct_email,
        doct_commission_rate,
        doct_other_commission_rate,
      ];

      db_con.query(queryInsert, values, (insertError, insertResults) => {
        if (insertError) {
          res.status(500).send(insertError);
        } else {
          // Call insertPlAcc function here after successfully inserting doctor data
          insertPlAcc(db_con, doct_id, "DPL", (error, result) => {
            if (error) {
              console.error("Error:", error);
              res.status(500).send(error);
            } else {
              res.status(201).json({
                message: "Doctor added successfully",
                doctorId: doct_id,
                message_2: "Tracker Account Created successfully",
                placcId: result.pl_acc_no,
              });
            }
          });
        }
      });
    });
  });
};

// ! Update doctor data query
const updateDoctor = (req, res) => {
  const doctorID = req.params.id;
  const {
    doct_name,
    doct_location,
    doct_contact,
    doct_email,
    doct_commission_rate,
    doct_other_commission_rate,
    doct_active_status,
  } = req.body;

  // ? Update doctor details
  const queryUpdate =
    "UPDATE sys_doctor_tbl SET doct_name = ?, doct_location = ?, doct_contact = ?, doct_email = ?, doct_commission_rate = ?, doct_other_commission_rate = ?, doct_active_status = ? WHERE doct_id = ?";
  const values = [
    doct_name,
    doct_location,
    doct_contact,
    doct_email,
    doct_commission_rate,
    doct_other_commission_rate,
    doct_active_status,
    doctorID,
  ];

  db_con.query(queryUpdate, values, (error, results) => {
    if (error) {
      res.status(500).send(error);
    } else {
      if (results.affectedRows > 0) {
        res.status(200).json({ message: "Doctor updated successfully" });
      } else {
        res.status(404).json({ message: "Doctor not found" });
      }
    }
  });
};

// ! Get all doctor data query (table view)
const getAllDoctors = (req, res) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;

    const selectQuery = `
          SELECT doct_id, doct_name, doct_location, doct_contact, doct_email, doct_commission_rate, doct_other_commission_rate, doct_active_status
          FROM sys_doctor_tbl
          WHERE doct_name LIKE ? OR doct_id LIKE ? OR doct_contact LIKE ? OR doct_location LIKE ? OR doct_email LIKE ?
          ORDER BY ${orderColumn} ${orderDirection}
          LIMIT ?, ?;
        `;

    const countQuery = `
          SELECT COUNT(doct_id) AS total FROM sys_doctor_tbl
          WHERE doct_name LIKE ? OR doct_id LIKE ? OR doct_contact LIKE ? OR doct_location LIKE ? OR doct_email LIKE ?
        `;

    db_con.query(
      selectQuery,
      [
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        offset,
        limit,
      ],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        // ? Check if there's no data in the table
        if (results.length === 0) {
          return res.status(200).json({
            draw: +draw,
            recordsTotal: 0,
            recordsFiltered: 0,
            data: [],
            message: "No data found in the table.",
          });
        }

        db_con.query(
          countQuery,
          [
            searchPattern,
            searchPattern,
            searchPattern,
            searchPattern,
            searchPattern,
          ],
          (countError, countResults) => {
            if (countError) {
              return res.status(500).send(countError);
            }

            const totalRecords = countResults[0].total;
            const response = {
              draw: +draw,
              recordsTotal: totalRecords,
              recordsFiltered: totalRecords,
              data: results,
            };

            res.status(200).json(response);
          }
        );
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

// ! Get doctor data query by selected doctor id
const getDoctorById = (req, res) => {
  const doctorID = req.params.id;

  // ? Doctor data get query
  db_con.query(
    "SELECT * FROM sys_doctor_tbl WHERE doct_id = ?",
    [doctorID],
    (error, results) => {
      if (error) {
        res.status(500).send(error);
      } else {
        if (results.length === 0) {
          res
            .status(404)
            .json({ message: "No data found for the specified doctor ID" });
        } else {
          res.status(200).json(results[0]);
        }
      }
    }
  );
};

// ! get all doctor names and id query
const getDoctorNamesAndID = (req, res) => {
  // ? doctor data get query
  db_con.query(
    "SELECT doct_id,doct_name,CONCAT(doct_name, ' - ', doct_location) AS show_name, pl_acc_avb_balance, CONCAT(doct_name, ' - ', doct_location, ' - Rs. ', pl_acc_avb_balance) AS show_name_2  FROM sys_doctor_tbl INNER JOIN pl_acc_tbl ON (sys_doctor_tbl.doct_id = pl_acc_tbl.pl_acc_tracker_id)",
    (error, results) => {
      if (error) {
        res.status(500).send(error);
      } else {
        if (results.length === 0) {
          // ? No data found for the specified doctor ID
          res
            .status(404)
            .json({ message: "No data found for the specified doctor ID" });
        } else {
          res.status(200).json(results);
        }
      }
    }
  );
};

module.exports = {
  createDoctor,
  updateDoctor,
  getAllDoctors,
  getDoctorById,
  getDoctorNamesAndID,
};
