const db_con = require("../../../config/database-connection");

// ! insert hospital data query
const createHospital = (req, res) => {
  const { hos_name, hos_location, hos_contact } = req.body;

  // ? Validate required fields
  if (!hos_name || !hos_location || !hos_contact) {
    return res.status(400).json({ error: "All fields are required" });
  }

  // ? Check if the hospital name already exists
  const queryCheckName =
    "SELECT COUNT(*) AS count FROM sys_hospital_tbl WHERE hos_name = ?";
  db_con.query(queryCheckName, [hos_name], (checkError, checkResults) => {
    if (checkError) {
      return res.status(500).send(checkError);
    }

    const nameExists = checkResults[0].count > 0;

    if (nameExists) {
      return res.status(400).json({ error: "Hospital name already exists" });
    }

    // ? Get the max from existing hospitals
    const queryMaxId =
      "SELECT MAX(CAST(SUBSTRING(hos_id,2) AS UNSIGNED)) AS max_id FROM sys_hospital_tbl";
    db_con.query(queryMaxId, (error, results) => {
      if (error) {
        return res.status(500).send(error);
      }

      const maxID = results[0].max_id || 0;
      const nextID = maxID + 1;

      // ? Format the next hospital id
      const hos_id = "H" + `${String(nextID).padStart(4, "0")}`;

      const queryInsert =
        "INSERT INTO sys_hospital_tbl (hos_id, hos_name, hos_location, hos_contact) VALUES (?, ?, ?, ?)";
      const values = [hos_id, hos_name, hos_location, hos_contact];

      db_con.query(queryInsert, values, (insertError, insertResults) => {
        if (insertError) {
          res.status(500).send(insertError);
        } else {
          res.status(201).json({
            message: "Hospital added successfully",
            hospitalId: hos_id,
          });
        }
      });
    });
  });
};

// ! update hospital data query
const updateHospital = (req, res) => {
  const hospitalID = req.params.id;
  const { hos_name, hos_location, hos_contact } = req.body;

  // ? Update hospital details
  const queryUpdate =
    "UPDATE sys_hospital_tbl SET hos_name = ?, hos_location = ?, hos_contact = ? WHERE hos_id = ?";
  const values = [hos_name, hos_location, hos_contact, hospitalID];

  db_con.query(queryUpdate, values, (error, results) => {
    if (error) {
      res.status(500).send(error);
    } else {
      if (results.affectedRows > 0) {
        res.status(200).json({ message: "Hospital updated successfully" });
      } else {
        res.status(404).json({ message: "Hospital not found" });
      }
    }
  });
};

// ! view all hospital data query (table view)
const getAllHospitals = (req, res) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;

    const selectQuery = `
      SELECT hos_id, hos_name, hos_location, hos_contact
      FROM sys_hospital_tbl
      WHERE hos_name LIKE ? OR hos_id LIKE ? OR hos_location LIKE ?
      ORDER BY ${orderColumn} ${orderDirection}
      LIMIT ?, ?
    `;

    const countQuery = `
      SELECT COUNT(hos_id) AS total FROM sys_hospital_tbl
      WHERE hos_name LIKE ? OR hos_id LIKE ? OR hos_location LIKE ?
    `;

    db_con.query(
      selectQuery,
      [searchPattern, searchPattern, searchPattern, offset, limit],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        db_con.query(
          countQuery,
          [searchPattern, searchPattern, , searchPattern],
          (countError, countResults) => {
            if (countError) {
              return res.status(500).send(countError);
            }

            const totalRecords = countResults[0].total;
            const response = {
              draw: +draw,
              recordsTotal: totalRecords,
              recordsFiltered: totalRecords,
              data: results,
            };

            res.status(200).json(response);
          }
        );
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

// ! get hospital data query selected hospital id
const getHospitalById = (req, res) => {
  const hospitalID = req.params.id;

  // ? hospital data get query
  db_con.query(
    "SELECT hos_id,hos_name, hos_location, hos_contact FROM sys_hospital_tbl WHERE hos_id = ?",
    [hospitalID],
    (error, results) => {
      if (error) {
        res.status(500).send(error);
      } else {
        if (results.length === 0) {
          // ? No data found for the specified hospital ID
          res
            .status(404)
            .json({ message: "No data found for the specified hospital ID" });
        } else {
          res.status(200).json(results[0]);
        }
      }
    }
  );
};

// ! get all hospital names and id query
const getHospitalNamesAndID = (req, res) => {
  // ? hospital data get query
  db_con.query(
    "SELECT hos_id,hos_name FROM sys_hospital_tbl",
    (error, results) => {
      if (error) {
        res.status(500).send(error);
      } else {
        if (results.length === 0) {
          // ? No data found for the specified hospital ID
          res
            .status(404)
            .json({ message: "No data found for the specified hospital ID" });
        } else {
          res.status(200).json(results);
        }
      }
    }
  );
};

module.exports = {
  createHospital,
  updateHospital,
  getAllHospitals,
  getHospitalById,
  getHospitalNamesAndID,
};
