const db_con = require("../../../config/database-connection");
const insertPlAcc = require("../../../Function/pl-acc-function");

// ! insert patient data query
const createPatient = (req, res) => {
  const {
    pat_name,
    pat_city,
    pat_contact,
    pat_email,
    pat_birthday,
    pat_gender,
    pat_branch,
  } = req.body;

  // ? Validate required fields
  if (!pat_name || !pat_city || !pat_contact || !pat_birthday) {
    return res.status(400).json({ error: "All fields are required" });
  }

  // ? Check if the patient name already exists
  const queryCheckName =
    "SELECT COUNT(*) AS count FROM sys_patient_tbl WHERE pat_name = ? AND pat_contact = ?";
  db_con.query(
    queryCheckName,
    [pat_name, pat_contact],
    (checkError, checkResults) => {
      if (checkError) {
        return res.status(500).send(checkError);
      }

      const nameExists = checkResults[0].count > 0;

      if (nameExists) {
        return res.status(400).json({ error: "patient name already exists" });
      }

      // ? Get the max from existing patients
      const queryMaxId =
        "SELECT MAX(CAST(SUBSTRING(pat_id,2) AS UNSIGNED)) AS max_id FROM sys_patient_tbl";
      db_con.query(queryMaxId, (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        const maxID = results[0].max_id || 0;
        const nextID = maxID + 1;

        // ? Format the next patient id
        const pat_id = "P" + `${String(nextID).padStart(5, "0")}`;

        const queryInsert =
          "INSERT INTO sys_patient_tbl (pat_id, pat_name, pat_city, pat_contact,pat_email,pat_birthday,pat_gender,pat_branch) VALUES (?, ?, ?, ?, ?, ?, ?, ?)";
        const values = [
          pat_id,
          pat_name,
          pat_city,
          pat_contact,
          pat_email,
          pat_birthday,
          pat_gender,
          pat_branch,
        ];

        db_con.query(queryInsert, values, (insertError, insertResults) => {
          if (insertError) {
            res.status(500).send(insertError);
          } else {
            // Call the stored procedure after the data is inserted
            const queryCallSP = "CALL SP_cal_patient_age()";
            db_con.query(queryCallSP, (spError, spResults) => {
              if (spError) {
                console.error("Error calling stored procedure:", spError);
              }

              // Call insertPlAcc function here after successfully inserting patient data
              insertPlAcc(db_con, pat_id, "CPL", (error, result) => {
                if (error) {
                  console.error("Error:", error);
                  res.status(500).send(error);
                } else {
                  res.status(201).json({
                    message: "patient added successfully",
                    patientId: pat_id,
                    message_2: "Tracker Account Created successfully",
                    placcId: result.pl_acc_no,
                  });
                }
              });
            });
          }
        });
      });
    }
  );
};

// ! update patient data query
const updatePatient = (req, res) => {
  const patientID = req.params.id;
  const {
    pat_name,
    pat_city,
    pat_contact,
    pat_email,
    pat_birthday,
    pat_gender,
  } = req.body;

  // Update patient details
  const queryUpdate =
    "UPDATE sys_patient_tbl SET pat_name = ?, pat_city = ?, pat_contact = ?, pat_email = ?, pat_birthday = ?, pat_gender = ? WHERE pat_id = ?";
  const values = [
    pat_name,
    pat_city,
    pat_contact,
    pat_email,
    pat_birthday,
    pat_gender,
    patientID,
  ];

  db_con.query(queryUpdate, values, (error, results) => {
    if (error) {
      res.status(500).send(error);
    } else {
      // Call the stored procedure after the data is updated
      const queryCallSP = "CALL SP_cal_patient_age()";
      db_con.query(queryCallSP, (spError, spResults) => {
        if (spError) {
          console.error("Error calling stored procedure:", spError);
          res.status(500).send("Error calling stored procedure");
        } else {
          if (results.affectedRows > 0) {
            res.status(200).json({ message: "Patient updated successfully" });
          } else {
            res.status(404).json({ message: "Patient not found" });
          }
        }
      });
    }
  });
};

// ! view all patient data query (table view)
const getAllPatients = (req, res) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;

    const selectQuery = `
        SELECT pat_id, pat_name, pat_city, pat_contact, pat_email, DATE_FORMAT(pat_birthday, '%Y-%m-%d') AS pat_birthday, pat_age,pat_gender
        FROM sys_patient_tbl
        WHERE pat_name LIKE ? OR pat_id LIKE ? OR pat_contact LIKE ? OR pat_birthday LIKE ? OR pat_city LIKE ?
        ORDER BY ${orderColumn} ${orderDirection}
        LIMIT ?, ?
      `;

    const countQuery = `
        SELECT COUNT(pat_id) AS total FROM sys_patient_tbl
        WHERE pat_name LIKE ? OR pat_id LIKE ? OR pat_contact LIKE ? OR pat_birthday LIKE ?  OR pat_city LIKE ?
      `;

    db_con.query(
      selectQuery,
      [
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        searchPattern,
        offset,
        limit,
      ],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        // Check if there's no data in the table
        if (results.length === 0) {
          return res.status(200).json({
            draw: +draw,
            recordsTotal: 0,
            recordsFiltered: 0,
            data: [],
            message: "No data found in the table.",
          });
        }

        db_con.query(
          countQuery,
          [
            searchPattern,
            searchPattern,
            searchPattern,
            searchPattern,
            searchPattern,
          ],
          (countError, countResults) => {
            if (countError) {
              return res.status(500).send(countError);
            }

            const totalRecords = countResults[0].total;
            const response = {
              draw: +draw,
              recordsTotal: totalRecords,
              recordsFiltered: totalRecords, // Use totalRecords for recordsFiltered
              data: results,
            };

            res.status(200).json(response);
          }
        );
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

// ! get patient data query selected patient id
const getPatientById = (req, res) => {
  const patientID = req.params.id;

  // ? patient data get query
  db_con.query(
    "SELECT pat_id,pat_name, pat_city, pat_contact,pat_email,DATE_FORMAT(pat_birthday, '%Y-%m-%d') AS pat_birthday,pat_age,pat_gender FROM sys_patient_tbl WHERE pat_id = ?",
    [patientID],
    (error, results) => {
      if (error) {
        res.status(500).send(error);
      } else {
        if (results.length === 0) {
          // ? No data found for the specified patient ID
          res
            .status(404)
            .json({ message: "No data found for the specified patient ID" });
        } else {
          res.status(200).json(results[0]);
        }
      }
    }
  );
};

// ! get all patient names and id query
const getPatientNamesAndID = (req, res) => {
  // ? patient data get query
  db_con.query(
    "SELECT pat_id,pat_name,pat_contact, CONCAT(pat_id, ' - ', pat_name, ' - ', pat_contact) AS show_name FROM sys_patient_tbl ORDER BY pat_id DESC",
    (error, results) => {
      if (error) {
        res.status(500).send(error);
      } else {
        if (results.length === 0) {
          // ? No data found for the specified patient ID
          res
            .status(404)
            .json({ message: "No data found for the specified patient ID" });
        } else {
          res.status(200).json(results);
        }
      }
    }
  );
};

module.exports = {
  createPatient,
  updatePatient,
  getAllPatients,
  getPatientById,
  getPatientNamesAndID,
};
