const db_con = require("../../../config/database-connection");
const bcrypt = require("bcryptjs");

const createUser = async (req, res) => {
  const {
    user_fname,
    user_lname,
    user_contact,
    user_password,
    user_create_date,
    user_update_date,
    user_type,
    user_gender,
  } = req.body;

  // ? Ensure that user_type is provided in the request body
  if (!user_type) {
    return res.status(400).json({ message: "User type is required" });
  }

  try {
    // ? Hash the password using bcrypt
    const hashedPassword = await bcrypt.hash(user_password, 10); // The second parameter is the saltRounds

    // ? Get the max user_id for the given user_type
    const queryMaxId = `SELECT MAX(CAST(SUBSTRING(user_id, 3) AS UNSIGNED)) AS max_id FROM sys_user_tbl WHERE user_type = ?`;
    db_con.query(queryMaxId, [user_type], (error, results) => {
      if (error) {
        return res.status(500).send(error);
      }

      const maxId = results[0].max_id || 0;
      const nextId = maxId + 1;

      // ? Format the next user_id
      const user_id = `${user_type}${String(nextId).padStart(4, "0")}`;

      // ? Insert the user into the table with the hashed password
      const queryInsert =
        "INSERT INTO sys_user_tbl (user_id, user_fname, user_lname, user_contact, user_password, user_create_date, user_update_date, user_type, user_gender) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)";
      const values = [
        user_id,
        user_fname,
        user_lname,
        user_contact,
        hashedPassword, // Use the hashed password
        user_create_date,
        user_update_date,
        user_type,
        user_gender,
      ];

      db_con.query(queryInsert, values, (insertError, insertResults) => {
        if (insertError) {
          res.status(500).send(insertError);
        } else {
          res.status(201).json({
            message: "User created successfully",
            userId: insertResults.insertId,
          });
        }
      });
    });
  } catch (error) {
    res.status(500).send(error);
  }
};

const updateUser = (req, res) => {
  const userId = req.params.id;
  const {
    user_fname,
    user_lname,
    user_contact,
    user_gender,
    user_update_date,
  } = req.body;

  const query =
    "UPDATE sys_user_tbl SET user_fname=?, user_lname=?, user_contact=?, user_gender=?, user_update_date=? WHERE user_id=?";
  const values = [
    user_fname,
    user_lname,
    user_contact,
    user_gender,
    user_update_date,
    userId,
  ];

  db_con.query(query, values, (error, results) => {
    if (error) {
      res.status(500).send(error);
    } else {
      if (results.affectedRows > 0) {
        res.status(200).json({ message: "User updated successfully" });
      } else {
        res.status(404).json({ message: "User not found" });
      }
    }
  });
};

const updateUserPassword = async (req, res) => {
  const userId = req.params.id;
  const { user_password, user_update_date } = req.body;

  // ? Hash the new password using bcrypt
  try {
    const hashedPassword = await bcrypt.hash(user_password, 10); // The second parameter is the saltRounds

    const query =
      "UPDATE sys_user_tbl SET user_password=?, user_update_date=? WHERE user_id=?";
    const values = [hashedPassword, user_update_date, userId];

    db_con.query(query, values, (error, results) => {
      if (error) {
        res.status(500).send(error);
      } else {
        if (results.affectedRows > 0) {
          res
            .status(200)
            .json({ message: "User password updated successfully" });
        } else {
          res.status(404).json({ message: "User not found" });
        }
      }
    });
  } catch (error) {
    res.status(500).send(error);
  }
};

const getAllUsers = (req, res) => {
  try {
    const { draw, start, length, search, order, columns } = req.query;
    const page = Math.ceil(start / length) + 1;
    const limit = +length;
    const offset = (page - 1) * limit;

    const orderColumnIndex = order[0].column;
    const orderDirection = order[0].dir;
    const orderColumn = columns[orderColumnIndex].data;

    const searchPattern = `%${search.value}%`;

    const selectQuery = `
      SELECT user_id, user_fname, user_lname, user_contact, user_create_date, user_update_date, user_type, user_gender 
      FROM sys_user_tbl
      WHERE user_fname LIKE ? OR user_lname LIKE ?
      ORDER BY ${orderColumn} ${orderDirection}
      LIMIT ?, ?
    `;

    const countQuery = `
      SELECT COUNT(user_id) AS total FROM sys_user_tbl
      WHERE user_fname LIKE ? OR user_lname LIKE ?
    `;

    db_con.query(
      selectQuery,
      [searchPattern, searchPattern, offset, limit],
      (error, results) => {
        if (error) {
          return res.status(500).send(error);
        }

        db_con.query(
          countQuery,
          [searchPattern, searchPattern],
          (countError, countResults) => {
            if (countError) {
              return res.status(500).send(countError);
            }

            const totalRecords = countResults[0].total;
            const response = {
              draw: +draw,
              recordsTotal: totalRecords,
              recordsFiltered: totalRecords, // Use totalRecords for recordsFiltered
              data: results,
            };

            res.status(200).json(response);
          }
        );
      }
    );
  } catch (error) {
    res.status(500).send(error.message);
  }
};

const getUserById = (req, res) => {
  const userId = req.params.id;
  db_con.query(
    "SELECT user_id,user_fname,user_lname,user_contact,user_create_date,user_update_date,user_type,user_gender FROM sys_user_tbl WHERE user_id = ?",
    [userId],
    (error, results) => {
      if (error) {
        res.status(500).send(error);
      } else {
        res.status(200).json(results[0]);
      }
    }
  );
};

const loginUser = (req, res) => {
  const { user_id, user_password } = req.body;

  // Retrieve the hashed password from the database
  const query =
    "SELECT user_id, user_password FROM sys_user_tbl WHERE user_id = ?";
  db_con.query(query, [user_id], async (error, results) => {
    if (error) {
      return res.status(500).send(error);
    }

    if (results.length === 0) {
      return res.status(401).json({ message: "Invalid username or password" });
    }

    const storedHashedPassword = results[0].user_password;

    try {
      // Compare the provided password with the stored hashed password using bcrypt
      const isMatch = await bcrypt.compare(user_password, storedHashedPassword);

      if (!isMatch) {
        return res
          .status(401)
          .json({ message: "Invalid username or password" });
      }

      // You can generate a token here and send it back as a response for successful login
      res.status(200).json({ message: "Login successful" });
    } catch (verifyError) {
      return res.status(500).send(verifyError);
    }
  });
};

module.exports = {
  getAllUsers,
  getUserById,
  createUser,
  updateUser,
  loginUser,
  updateUserPassword,
};
